# -*- mode: makefile; coding: utf-8 -*-
# Copyright © 2003 Colin Walters <walters@debian.org>
# Copyright © 2003,2008-2010, 2012-2016 Jonas Smedegaard <dr@jones.dk>
# Description: Defines useful variables for Perl modules
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 3, or (at your option)
# any later version.
#
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

# TODO: rewrite to use $(cdbs_make_curdestdir)

_cdbs_scripts_path ?= /usr/libexec/cdbs
_cdbs_rules_path ?= /usr/share/cdbs/1/rules
_cdbs_class_path ?= /usr/share/cdbs/1/class

ifndef _cdbs_class_perlmodule_vars
_cdbs_class_perlmodule_vars = 1

include $(_cdbs_class_path)/makefile-vars.mk$(_cdbs_makefile_suffix)

$(call cdbs_warn_deprecated,perlmodule-vars.mk,0.4.72,perl-makemaker-vars.mk)

cdbs_perl_ccflags := $(shell perl -MConfig -e 'print $$Config{ccflags}')
cdbs_perl_cppflags := $(shell perl -MConfig -e 'print $$Config{cppflags}')
cdbs_perl_lddlflags := $(shell perl -MConfig -e 'print $$Config{lddlflags}')

# Override optimizations to follow Perl Policy 3.9.4 § 4.3
# FIXME: Restructure to allow early override
DEB_MAKE_EXTRA_ARGS = \
	OPTIMIZE="$(strip \
		$(or $(CFLAGS_$(cdbs_curpkg)),$(CFLAGS)) \
		$(cdbs_perl_ccflags) \
		$(or $(CPPFLAGS_$(cdbs_curpkg)),$(CPPFLAGS)) \
		$(cdbs_perl_cppflags))" \
	LDDLFLAGS="$(strip \
		$(or $(LDFLAGS_$(cdbs_curpkg)),$(LDFLAGS)) \
		$(cdbs_perl_lddlflags))" \
	$(DEB_MAKE_PARALLEL)

# Install into first listed package by default.
# Unset for standard debhelper rules (use debian/tmp if multiple packages).
DEB_MAKEMAKER_PACKAGE ?= $(firstword \
	$(if $(_cdbs_rules_debhelper),\
		$(shell dh_listpackages),\
		$(shell $(_cdbs_scripts_path)/list-packages)))

DEB_MAKEMAKER_INVOKE ?= $(strip \
	/usr/bin/perl -I. Makefile.PL \
	$(DEB_MAKEMAKER_USER_FLAGS) \
	INSTALLDIRS=vendor)

# Set some MakeMaker defaults
# FIXME: Restructure to allow early override
DEB_MAKE_BUILD_TARGET = all
DEB_MAKE_CLEAN_TARGET = distclean
DEB_MAKE_CHECK_TARGET = test
DEB_MAKE_INSTALL_TARGET = install DESTDIR=$(strip \
	$(if $(DEB_MAKEMAKER_PACKAGE),\
		$(CURDIR)/debian/$(DEB_MAKEMAKER_PACKAGE),\
		$(DEB_DESTDIR)))

# Avoid auto-building dependencies
export PERL_AUTOINSTALL = --skipdeps

endif
